
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#ifndef FR_ENUM_TYPES_H
#define FR_ENUM_TYPES_H

#include <glib-object.h>

G_BEGIN_DECLS
/* enumerations from "typedefs.h" */
GType fr_compression_get_type (void);
#define FR_TYPE_COMPRESSION (fr_compression_get_type())
GType fr_overwrite_get_type (void);
#define FR_TYPE_OVERWRITE (fr_overwrite_get_type())
G_END_DECLS

#endif /* FR_ENUM_TYPES_H */

/* Generated data ends here */

