/* This file is generated by glib-genmarshal, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */
#ifndef __FR_MARSHAL_MARSHAL_H__
#define __FR_MARSHAL_MARSHAL_H__

#include <glib-object.h>

G_BEGIN_DECLS

/* VOID:INT (fr-marshal.list:1) */
#define fr_marshal_VOID__INT	g_cclosure_marshal_VOID__INT

/* VOID:INT,INT (fr-marshal.list:2) */
extern
void fr_marshal_VOID__INT_INT (GClosure     *closure,
                               GValue       *return_value,
                               guint         n_param_values,
                               const GValue *param_values,
                               gpointer      invocation_hint,
                               gpointer      marshal_data);

/* VOID:INT,BOXED (fr-marshal.list:3) */
extern
void fr_marshal_VOID__INT_BOXED (GClosure     *closure,
                                 GValue       *return_value,
                                 guint         n_param_values,
                                 const GValue *param_values,
                                 gpointer      invocation_hint,
                                 gpointer      marshal_data);

/* VOID:POINTER (fr-marshal.list:4) */
#define fr_marshal_VOID__POINTER	g_cclosure_marshal_VOID__POINTER

/* VOID:VOID (fr-marshal.list:5) */
#define fr_marshal_VOID__VOID	g_cclosure_marshal_VOID__VOID

/* VOID:DOUBLE (fr-marshal.list:6) */
#define fr_marshal_VOID__DOUBLE	g_cclosure_marshal_VOID__DOUBLE

/* VOID:DOUBLE,STRING (fr-marshal.list:7) */
extern
void fr_marshal_VOID__DOUBLE_STRING (GClosure     *closure,
                                     GValue       *return_value,
                                     guint         n_param_values,
                                     const GValue *param_values,
                                     gpointer      invocation_hint,
                                     gpointer      marshal_data);

/* VOID:STRING (fr-marshal.list:8) */
#define fr_marshal_VOID__STRING	g_cclosure_marshal_VOID__STRING

/* VOID:BOOL (fr-marshal.list:9) */
#define fr_marshal_VOID__BOOLEAN	g_cclosure_marshal_VOID__BOOLEAN

#define fr_marshal_VOID__BOOL	fr_marshal_VOID__BOOLEAN

/* VOID:BOXED (fr-marshal.list:10) */
#define fr_marshal_VOID__BOXED	g_cclosure_marshal_VOID__BOXED


G_END_DECLS

#endif /* __FR_MARSHAL_MARSHAL_H__ */
