// Generated by extract_c_strs.py. Do not edit!!!
#if 0
#include <QObject>

// ./src/core/fr-init.c
QObject::tr("7-Zip (.7z)");
QObject::tr("Tar compressed with 7z (.tar.7z)");
QObject::tr("Ace (.ace)");
QObject::tr("Ar (.a)");
QObject::tr("Ar (.ar)");
QObject::tr("Arj (.arj)");
QObject::tr("Tar compressed with bzip2 (.tar.bz2)");
QObject::tr("Tar compressed with bzip (.tar.bz)");
QObject::tr("Cabinet (.cab)");
QObject::tr("Rar Archived Comic Book (.cbr)");
QObject::tr("Zip Archived Comic Book (.cbz)");
QObject::tr("Tar compressed with gzip (.tar.gz)");
QObject::tr("Ear (.ear)");
QObject::tr("Self-extracting zip (.exe)");
QObject::tr("Jar (.jar)");
QObject::tr("Lha (.lzh)");
QObject::tr("Lrzip (.lrz)");
QObject::tr("Tar compressed with lrzip (.tar.lrz)");
QObject::tr("Tar compressed with lzip (.tar.lz)");
QObject::tr("Tar compressed with lzma (.tar.lzma)");
QObject::tr("Tar compressed with lzop (.tar.lzo)");
QObject::tr("Windows Imaging Format (.wim)");
QObject::tr("Rar (.rar)");
QObject::tr("Tar uncompressed (.tar)");
QObject::tr("Tar compressed with compress (.tar.Z)");
QObject::tr("War (.war)");
QObject::tr("Xz (.xz)");
QObject::tr("Tar compressed with xz (.tar.xz)");
QObject::tr("Tar compressed with zstd (.tar.zst)");
QObject::tr("Zoo (.zoo)");
QObject::tr("Zip (.zip)");

// ./src/core/fr-command-7z.c
QObject::tr("Adding file: ");
QObject::tr("Extracting file: ");

// ./src/core/glib-utils.c
QObject::tr("%d %B %Y, %H:%M");

// ./src/core/fr-command-tar.c
QObject::tr("Adding file: ");
QObject::tr("Removing file: ");
QObject::tr("Deleting files from archive");
QObject::tr("Extracting file: ");
QObject::tr("Recompressing archive");
QObject::tr("Decompressing archive");

// ./src/core/fr-archive.c
QObject::tr("File not found.");
QObject::tr("Archive type not supported.");
QObject::tr("Archive not found");
QObject::tr("You don't have the right permissions.");
QObject::tr("This archive type cannot be modified");
QObject::tr("You can't add an archive to itself.");

// ./src/core/fr-command-rar.c
QObject::tr("Adding file: ");
QObject::tr("Removing file: ");
QObject::tr("Extracting file: ");
QObject::tr("Could not find the volume: %s");


#endif

